<?php
	// If this file is called directly, abort.
	if ( ! defined( 'WPINC' ) ) {
		die;
	}


	/**
	 * The file that defines the core plugin class
	 *
	 * A class definition that includes attributes and functions used across both the
	 * public-facing side of the site and the admin area.
	 *
	 * @link       https://codeboxr.com
	 * @since      1.0.0
	 *
	 * @package    CBCurrencyConverter
	 * @subpackage CBCurrencyConverter/includes
	 */

	/**
	 * The core plugin class.
	 *
	 * This is used to define internationalization, admin-specific hooks, and
	 * public-facing site hooks.
	 *
	 * Also maintains the unique identifier of this plugin as well as the current
	 * version of the plugin.
	 *
	 * @since      1.0.0
	 * @package    CBCurrencyConverter
	 * @subpackage CBCurrencyConverter/includes
	 * @author     codeboxr <info@codeboxr.com>
	 */
	class CBCurrencyConverter {

		/**
		 * The loader that's responsible for maintaining and registering all hooks that power
		 * the plugin.
		 *
		 * @since    1.0.0
		 * @access   protected
		 * @var      CBCurrencyConverter_Loader $loader Maintains and registers all hooks for the plugin.
		 */
		protected $loader;

		/**
		 * The unique identifier of this plugin.
		 *
		 * @since    1.0.0
		 * @access   protected
		 * @var      string $plugin_name The string used to uniquely identify this plugin.
		 */
		protected $plugin_name;

		public $plugin_basename;

		/**
		 * The current version of the plugin.
		 *
		 * @since    1.0.0
		 * @access   protected
		 * @var      string $version The current version of the plugin.
		 */
		protected $version;

		/**
		 * Define the core functionality of the plugin.
		 *
		 * Set the plugin name and the plugin version that can be used throughout the plugin.
		 * Load the dependencies, define the locale, and set the hooks for the admin area and
		 * the public-facing side of the site.
		 *
		 * @since    1.0.0
		 */
		public function __construct() {

			$this->plugin_name = CBCURRENCYCONVERTER_NAME;
			$this->version     = CBCURRENCYCONVERTER_VERSION;

			$this->plugin_basename = plugin_basename( plugin_dir_path( __DIR__ ) . $this->plugin_name . '.php' );

			$this->load_dependencies();
			$this->set_locale();
			$this->define_admin_hooks();
			$this->define_public_hooks();

		}

		/**
		 * Get currency list
		 *
		 * @return mixed|void
		 */
		public static function getCurrencyList() {
			cbcurrencyconverter_deprecated_function( 'getCurrencyList function', '2.2', 'CBCurrencyConverterHelper::getCurrencyList' );

			return CBCurrencyConverterHelper::getCurrencyList();
		}

		/**
		 * Load the required dependencies for this plugin.
		 *
		 * Include the following files that make up the plugin:
		 *
		 * - CBCurrencyConverter_Loader. Orchestrates the hooks of the plugin.
		 * - CBCurrencyConverter_i18n. Defines internationalization functionality.
		 * - CBCurrencyConverter_Admin. Defines all hooks for the admin area.
		 * - CBCurrencyConverter_Public. Defines all hooks for the public side of the site.
		 *
		 * Create an instance of the loader which will be used to register the hooks
		 * with WordPress.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function load_dependencies() {

			/**
			 * The class responsible for orchestrating the actions and filters of the
			 * core plugin.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbcurrencyconverter-loader.php';

			/**
			 * The class responsible for defining internationalization functionality
			 * of the plugin.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbcurrencyconverter-i18n.php';


			/**
			 * The class responsible for settings in admin area.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbcurrencyconverter-setting.php';


			/**
			 * Includer helper function
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbcurrencyconverter-helper.php';

			/**
			 * The class responsible for defining all actions that occur in the admin area.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-cbcurrencyconverter-admin.php';

			/**
			 * The class responsible for defining all actions that occur in the public-facing
			 * side of the site.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-cbcurrencyconverter-public.php';

			/**
			 * Register Widget
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'widgets/classic_widgets/class-cbcurrencyconverter-widget.php';

			$this->loader = new CBCurrencyConverter_Loader();

		}

		/**
		 * Define the locale for this plugin for internationalization.
		 *
		 * Uses the CBCurrencyConverter_i18n class in order to set the domain and to register the hook
		 * with WordPress.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function set_locale() {

			$plugin_i18n = new CBCurrencyConverter_i18n();
			$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
		}//end set_locale

		/**
		 * Register all of the hooks related to the admin area functionality
		 * of the plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function define_admin_hooks() {
			$plugin_admin = new CBCurrencyConverter_Admin( $this->get_plugin_name(), $this->get_version() );

			$this->loader->add_filter( 'plugin_action_links_' . CBCURRENCYCONVERTER_BASE_NAME, $plugin_admin, 'action_links', 10, 4 );
			$this->loader->add_filter( 'plugin_row_meta', $plugin_admin, 'custom_plugin_row_meta', 10, 2 );
			$this->loader->add_action( 'upgrader_process_complete', $plugin_admin, 'plugin_upgrader_process_complete', 10, 2 );
			$this->loader->add_action( 'admin_notices', $plugin_admin, 'plugin_activate_upgrade_notices' );

			$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
			$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

			$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_menus' );
			$this->loader->add_action( 'admin_init', $plugin_admin, 'setting_init' );

			//full reset of options created by this plugin
			if ( isset( $_REQUEST['page'] ) && $_REQUEST['page'] == 'cbcurrencyconverter' && isset( $_REQUEST['cbcurrencyconverter_fullreset'] ) && $_REQUEST['cbcurrencyconverter_fullreset'] == 1 ) {
				$this->loader->add_action( 'admin_init', $plugin_admin, 'plugin_fullreset' );
			}

			//full reset of transient caches created by this plugin
			if ( isset( $_REQUEST['page'] ) && $_REQUEST['page'] == 'cbcurrencyconverter' && isset( $_REQUEST['cbcurrencyconverter_transientreset'] ) && $_REQUEST['cbcurrencyconverter_transientreset'] == 1 ) {
				$this->loader->add_action( 'admin_init', $plugin_admin, 'plugin_transientreset' );
			}


			//gutenberg
			$this->loader->add_action( 'init', $plugin_admin, 'gutenberg_blocks' );
			$this->loader->add_filter( 'block_categories', $plugin_admin, 'gutenberg_block_categories', 10, 2 );
			$this->loader->add_action( 'enqueue_block_editor_assets', $plugin_admin, 'enqueue_block_editor_assets' );


            //update manager
            $this->loader->add_filter('pre_set_site_transient_update_plugins', $plugin_admin, 'pre_set_site_transient_update_plugins_pro_addon');
            $this->loader->add_action( 'in_plugin_update_message-' . 'cbcurrencyconverteraddon/cbcurrencyconverteraddon.php', $plugin_admin, 'plugin_update_message_pro_addons' );


			do_action( 'cbcurrencyconverter_admin', $plugin_admin );
		}//end define_admin_hooks

		/**
		 * Register all of the hooks related to the public-facing functionality
		 * of the plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function define_public_hooks() {
			global $plugin_public;

			$plugin_public = new CBCurrencyConverter_Public( $this->get_plugin_name(), $this->get_version() );

			$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
			$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

			//init widget
			$this->loader->add_action( 'widgets_init', $plugin_public, 'register_widgets' );

			//shortcode
			$this->loader->add_action( 'init', $plugin_public, 'init_shortcodes' );

			$this->loader->add_filter( 'cbxcc_convertion_method', $plugin_public, 'cbxconvertcurrency_method_alphavantage', 10, 5 );
			//$this->loader->add_filter( 'cbxcc_convertion_method', $plugin_public, 'cbxconvertcurrency_method_switcher', 10, 5 );

			$this->loader->add_action( 'wp_ajax_currrency_convert', $plugin_public, 'cbcurrencyconverter_ajax_cur_convert' );
			$this->loader->add_action( 'wp_ajax_nopriv_currrency_convert', $plugin_public, 'cbcurrencyconverter_ajax_cur_convert' );

			$this->loader->add_action( 'init', $plugin_public, 'cbcurrencyconverter_init' );

			//elementor
			//$this->loader->add_action( 'elementor/init', $plugin_public, 'init_elementor_widgets' );
			$this->loader->add_action( 'elementor/widgets/widgets_registered', $plugin_public, 'init_elementor_widgets' );
			$this->loader->add_action( 'elementor/elements/categories_registered', $plugin_public, 'add_elementor_widget_categories' );
			$this->loader->add_action( 'elementor/editor/before_enqueue_scripts', $plugin_public, 'elementor_icon_loader', 99999 );

			$this->loader->add_action( 'vc_before_init', $plugin_public, 'vc_before_init_actions', 12 );//priority changed from default(10) to 12
		}//end define_public_hooks

		/**
		 * Run the loader to execute all of the hooks with WordPress.
		 *
		 * @since    1.0.0
		 */
		public function run() {
			$this->loader->run();
		}

		/**
		 * The name of the plugin used to uniquely identify it within the context of
		 * WordPress and to define internationalization functionality.
		 *
		 * @return    string    The name of the plugin.
		 * @since     1.0.0
		 */
		public function get_plugin_name() {
			return $this->plugin_name;
		}

		/**
		 * The reference to the class that orchestrates the hooks with the plugin.
		 *
		 * @return    CBCurrencyConverter_Loader    Orchestrates the hooks of the plugin.
		 * @since     1.0.0
		 */
		public function get_loader() {
			return $this->loader;
		}

		/**
		 * Retrieve the version number of the plugin.
		 *
		 * @return    string    The version number of the plugin.
		 * @since     1.0.0
		 */
		public function get_version() {
			return $this->version;
		}
	}//end class CBCurrencyConverter